@@ -16,7 +16,7 @@ portage.proxy.lazyimport.lazyimport(globals(),
 	'portage.package.ebuild.doebuild:doebuild',
 	'portage.util:ensure_dirs,shlex_split,writemsg,writemsg_level',
 	'portage.util.listdir:listdir',
-	'portage.versions:best,catpkgsplit,_pkgsplit@pkgsplit,ver_regexp,_pkg_str',
+	'portage.versions:best,catsplit,catpkgsplit,_pkgsplit@pkgsplit,ver_regexp,_pkg_str',
 )
 
 from portage.cache import volatile
@@ -43,6 +43,8 @@ import os as _os
 import sys
 import traceback
 import warnings
+import errno
+import collections
 
 try:
 	from urllib.parse import urlparse
@@ -101,6 +103,68 @@ class _dummy_list(list):
 		except ValueError:
 			pass
 
+
+class _better_cache(object):
+
+	"""
+	The purpose of better_cache is to locate catpkgs in repositories using ``os.listdir()`` as much as possible, which
+	is less expensive IO-wise than exhaustively doing a stat on each repo for a particular catpkg. better_cache stores a
+	list of repos in which particular catpkgs appear. Various dbapi methods use better_cache to locate repositories of
+	interest related to particular catpkg rather than performing an exhaustive scan of all repos/overlays.
+
+	Better_cache.items data may look like this::
+
+	  { "sys-apps/portage" : [ repo1, repo2 ] }
+
+	Without better_cache, Portage will get slower and slower (due to excessive IO) as more overlays are added.
+
+	Also note that it is OK if this cache has some 'false positive' catpkgs in it. We use it to search for specific
+	catpkgs listed in ebuilds. The likelihood of a false positive catpkg in our cache causing a problem is extremely
+	low, because the user of our cache is passing us a catpkg that came from somewhere and has already undergone some
+	validation, and even then will further interrogate the short-list of repos we return to gather more information
+	on the catpkg.
+
+	Thus, the code below is optimized for speed rather than painstaking correctness. I have added a note to
+	``dbapi.getRepositories()`` to ensure that developers are aware of this just in case.
+
+	The better_cache has been redesigned to perform on-demand scans -- it will only scan a category at a time, as
+	needed. This should further optimize IO performance by not scanning category directories that are not needed by
+	Portage.
+	"""
+
+	def __init__(self, repositories):
+		self._items = collections.defaultdict(list)
+		self._scanned_cats = set()
+
+		# ordered list of all portree locations we'll scan:
+		self._repo_list = [repo for repo in reversed(list(repositories))
+			if repo.location is not None]
+
+	def __getitem__(self, catpkg):
+		result = self._items.get(catpkg)
+		if result is not None:
+			return result
+
+		cat, pkg = catsplit(catpkg)
+		if cat not in self._scanned_cats:
+			self._scan_cat(cat)
+		return self._items[catpkg]
+
+	def _scan_cat(self, cat):
+		for repo in self._repo_list:
+			cat_dir = repo.location + "/" + cat
+			try:
+				pkg_list = os.listdir(cat_dir)
+			except OSError as e:
+				if e.errno not in (errno.ENOTDIR, errno.ENOENT, errno.ESTALE):
+					raise
+				continue
+			for p in pkg_list:
+				if os.path.isdir(cat_dir + "/" + p):
+					self._items[cat + "/" + p].append(repo)
+		self._scanned_cats.add(cat)
+
+
 class portdbapi(dbapi):
 	"""this tree will scan a portage directory located at root (passed to init)"""
 	portdbapi_instances = _dummy_list()
@@ -253,6 +317,7 @@ class portdbapi(dbapi):
 			"RESTRICT", "SLOT", "DEFINED_PHASES", "REQUIRED_USE"])
 
 		self._aux_cache = {}
+		self._better_cache = None
 		self._broken_ebuilds = set()
 
 	@property
@@ -342,12 +407,25 @@ class portdbapi(dbapi):
 		except KeyError:
 			return None
 
-	def getRepositories(self):
+	def getRepositories(self, catpkg=None):
+
 		"""
-		This function is required for GLEP 42 compliance; it will return a list of
-		repository IDs
-		TreeMap = {id: path}
+		With catpkg=None, this will return a complete list of repositories in this dbapi. With catpkg set to a value,
+		this method will return a short-list of repositories that contain this catpkg. Use this second approach if
+		possible, to avoid exhaustively searching all repos for a particular catpkg. It's faster for this method to
+		find the catpkg than for you do it yourself. When specifying catpkg, you should have reasonable assurance that
+		the category is valid and PMS-compliant as the caching mechanism we use does not perform validation checks for
+		categories.
+
+		This function is required for GLEP 42 compliance.
+
+		@param catpkg: catpkg for which we want a list of repositories; we'll get a list of all repos containing this
+		  catpkg; if None, return a list of all Repositories that contain a particular catpkg.
+		@return: a list of repositories.
 		"""
+
+		if catpkg is not None and self._better_cache is not None:
+			return [repo.name for repo in self._better_cache[catpkg]]
 		return self._ordered_repo_name_list
 
 	def getMissingRepoNames(self):
@@ -363,7 +441,7 @@ class portdbapi(dbapi):
 		"""
 		return self.settings.repositories.ignored_repos
 
-	def findname2(self, mycpv, mytree=None, myrepo = None):
+	def findname2(self, mycpv, mytree=None, myrepo=None):
 		""" 
 		Returns the location of the CPV, and what overlay it was in.
 		Searches overlays first, then PORTDIR; this allows us to return the first
@@ -385,16 +463,33 @@ class portdbapi(dbapi):
 		if psplit is None or len(mysplit) != 2:
 			raise InvalidPackageName(mycpv)
 
+		try:
+			cp = mycpv.cp
+		except AttributeError:
+			cp = mysplit[0] + "/" + psplit[0]
+
+		if self._better_cache is None:
+			if mytree:
+				mytrees = [mytree]
+			else:
+				mytrees = reversed(self.porttrees)
+		else:
+			try:
+				repos = self._better_cache[cp]
+			except KeyError:
+				return (None, 0)
+
+			mytrees = []
+			for repo in repos:
+				if mytree is not None and mytree != repo.location:
+					continue
+				mytrees.append(repo.location)
+
 		# For optimal performace in this hot spot, we do manual unicode
 		# handling here instead of using the wrapped os module.
 		encoding = _encodings['fs']
 		errors = 'strict'
 
-		if mytree:
-			mytrees = [mytree]
-		else:
-			mytrees = reversed(self.porttrees)
-
 		relative_path = mysplit[0] + _os.sep + psplit[0] + _os.sep + \
 			mysplit[1] + ".ebuild"
 
@@ -764,8 +859,10 @@ class portdbapi(dbapi):
 			else:
 				# assume it's iterable
 				mytrees = mytree
-		else:
+		elif self._better_cache is None:
 			mytrees = self.porttrees
+		else:
+			mytrees = [repo.location for repo in self._better_cache[mycp]]
 		for oroot in mytrees:
 			try:
 				file_list = os.listdir(os.path.join(oroot, mycp))
@@ -814,10 +911,12 @@ class portdbapi(dbapi):
 			"minimum-all-ignore-profile", "minimum-visible"):
 			self.xcache[x]={}
 		self.frozen=1
+		self._better_cache = _better_cache(self.repositories)
 
 	def melt(self):
 		self.xcache = {}
 		self._aux_cache = {}
+		self._better_cache = None
 		self.frozen = 0
 
 	def xmatch(self,level,origdep,mydep=None,mykey=None,mylist=None):
